package quad_tree;

import java.awt.*;

public class QTLeaf implements QTNode {

    private Point topLeft;
    private Point bottomRight;
    private int depth;

    private QTIntl parent;
    private Point point;

    public QTLeaf(Point topLeft, Point bottomRight) {
        this.topLeft = topLeft;
        this.bottomRight = bottomRight;
    }

    public QTLeaf(Point topLeft, Point bottomRight, Point point) {
        this(topLeft, bottomRight);
        this.point = point;
    }

    @Override
    public boolean isLeaf() {
        return true;
    }

    @Override
    public QTNode insert(Point p) {
        if (point == null) {
            point = p;
            return this;
        }
        QTNode asInternal = new QTIntl(topLeft, bottomRight);
        asInternal = asInternal.insert(point);
        asInternal = asInternal.insert(p);

        return asInternal;
    }

    public Point getTopLeft() {
        return topLeft;
    }

    public void setTopLeft(Point topLeft) {
        this.topLeft = topLeft;
    }

    public Point getBottomRight() {
        return bottomRight;
    }

    public void setBottomRight(Point bottomRight) {
        this.bottomRight = bottomRight;
    }

    public int getDepth() {
        return depth;
    }

    public void setDepth(int depth) {
        this.depth = depth;
    }

    public QTIntl getParent() {
        return parent;
    }

    public void setParent(QTIntl parent) {
        this.parent = parent;
    }

    public Point getPoint() {
        return point;
    }

    public void setPoint(Point point) {
        this.point = point;
    }
}

public class QTIntl implements QTNode {

    private Point topLeft;
    private Point bottomRight;
    private int depth;

    private QTNode topLeftTree;
    private QTNode topRightTree;
    private QTNode botLeftTree;
    private QTNode botRightTree;

    private quad_tree.QTIntl parent;

    public QTIntl(Point topLeft, Point bottomRight) {
        this.topLeft = topLeft;
        this.bottomRight = bottomRight;

        Point center = new Point(topLeft.x + ((bottomRight.x - topLeft.x) / 2), topLeft.y + ((bottomRight.y - topLeft.y) / 2));

        topLeftTree = new quad_tree.QTLeaf(topLeft, center);
        topRightTree = new quad_tree.QTLeaf(new Point(center.x, topLeft.y), new Point(bottomRight.x, center.y));
        botLeftTree = new quad_tree.QTLeaf(new Point(topLeft.x, center.y), new Point(center.x, bottomRight.y));
        botRightTree = new quad_tree.QTLeaf(center, bottomRight);

    }

    private Direction locate(Point p) {
        double xCenter = (bottomRight.x - topLeft.x) / 2.0;
        double yCenter = (bottomRight.y - topLeft.y) / 2.0;
        if (p.x < xCenter && p.y < yCenter) {
            return Direction.NW;
        } else if (p.x < xCenter && p.y >= yCenter) {
            return Direction.SW;
        } else if (p.x >= xCenter && p.y < yCenter) {
            return Direction.NE;
        } else {
            return Direction.SE;
        }
    }

    public boolean isLeaf() {
        return false;
    }

    @Override
    public QTNode insert(Point p) {
        switch (locate(p)) {
            case Direction.NW:
                topLeftTree = topLeftTree.insert(p);
                break;
            case Direction.NE:
                topRightTree = topRightTree.insert(p);
                break;
            case Direction.SW:
                botLeftTree = botLeftTree.insert(p);
                break;
            case Direction.SE:
                botRightTree = botRightTree.insert(p);
                break;
        }
        return this;
    }

    public Point getTopLeft() {
        return topLeft;
    }

    public void setTopLeft(Point topLeft) {
        this.topLeft = topLeft;
    }

    public Point getBottomRight() {
        return bottomRight;
    }

    public void setBottomRight(Point bottomRight) {
        this.bottomRight = bottomRight;
    }

    public int getDepth() {
        return depth;
    }

    public void setDepth(int depth) {
        this.depth = depth;
    }

    public QTNode getTopLeftTree() {
        return topLeftTree;
    }

    public void setTopLeftTree(QTNode topLeftTree) {
        this.topLeftTree = topLeftTree;
    }

    public QTNode getTopRightTree() {
        return topRightTree;
    }

    public void setTopRightTree(QTNode topRightTree) {
        this.topRightTree = topRightTree;
    }

    public QTNode getBotLeftTree() {
        return botLeftTree;
    }

    public void setBotLeftTree(QTNode botLeftTree) {
        this.botLeftTree = botLeftTree;
    }

    public QTNode getBotRightTree() {
        return botRightTree;
    }

    public void setBotRightTree(QTNode botRightTree) {
        this.botRightTree = botRightTree;
    }

    public quad_tree.QTIntl getParent() {
        return parent;
    }

    public void setParent(quad_tree.QTIntl parent) {
        this.parent = parent;
    }
}
